#include <imgui.h>
#include <rlImGui.h>
#include <rlgl.h>

#include "core/Panda.hpp"
#include "raylib.h"

#if PLATFORM_ANDROID

#include <imgui_impl_android.h>
#include <imgui_impl_opengl3.h>
#include <android_native_app_glue.h>

extern "C" struct android_app *GetAndroidApp(void);

static int32_t handleInputEvent(struct android_app* app, AInputEvent* inputEvent)
{
    return ImGui_ImplAndroid_HandleInputEvent(inputEvent);
}
#endif

void panda::GuiInit()
{
    #if PLATFORM_ANDROID
        GetAndroidApp()->onInputEvent = handleInputEvent;

        IMGUI_CHECKVERSION();
        ImGui::CreateContext();
        ImGuiIO& io = ImGui::GetIO();
        io.IniFilename = NULL;

        ImGui::StyleColorsDark();
        ImGui_ImplAndroid_Init(GetAndroidApp()->window);
        ImGui_ImplOpenGL3_Init("#version 300 es");

        ImFontConfig font_cfg;
        font_cfg.SizePixels = 22.0f;
        io.Fonts->AddFontDefault(&font_cfg);
        ImGui::GetStyle().ScaleAllSizes(3.0f);
    #else
        rlImGuiSetup(true);
    #endif
}

void panda::GuiDestroy()
{
    #if PLATFORM_ANDROID
        ImGui_ImplOpenGL3_Shutdown();
        ImGui_ImplAndroid_Shutdown();
        ImGui::DestroyContext();
    #else
        rlImGuiShutdown();
    #endif
}

void panda::GuiBegin()
{
    #if PLATFORM_ANDROID
        ImGui_ImplOpenGL3_NewFrame();
        ImGui_ImplAndroid_NewFrame();
        ImGui::NewFrame();
    #else
        rlImGuiBegin();
    #endif
}

void panda::GuiEnd()
{
    #if PLATFORM_ANDROID
        rlDrawRenderBatchActive();
        rlDisableBackfaceCulling();

        ImGui::Render();
        ImGui_ImplOpenGL3_RenderDrawData(ImGui::GetDrawData());

        rlSetTexture(0);
        rlEnableBackfaceCulling();
    #else
        rlImGuiEnd();
    #endif
}

void panda::DrawRenderTexture(const RenderTexture &buffer, const float &x, const float &y, const Color &color)
{
    DrawTextureRec(
        buffer.texture,
        {0.0f, 0.0f, 1.0f*buffer.texture.width, -1.0f*buffer.texture.height},
        {x, y}, color);
}

/////////////////////////////////




/////////////////////////////////

// panda::TilemapMetadata panda::LoadTilemapMetadata(const char *file)
// {
//     ini::IniFile ini;
//     char* text = LoadFileText(file);
//     ini.decode(text);

//     return TilemapMetadata{
//         ini["chunk"]["width"].as<int>(),
//         ini["chunk"]["height"].as<int>(),
//         ini["tile"]["width"].as<int>(),
//         ini["tile"]["height"].as<int>(),
//     };
// }

// panda::TilemapItem panda::LoadTilemap(const char *file, panda::TilemapMetadata metadata, Texture texture)
// {
//     return TilemapItem();
// }
