#pragma once

#include <memory>
#include <string>
#include <vector>
#include <unordered_map>

#include "core/Scene.hpp"
#include "utils/Random.hpp"

struct Texture;
struct RenderTexture;
struct Color;
struct Vector2;

namespace panda
{

    struct TextureAtlasItem
    {
        int x;
        int y;
        int w;
        int h;
        float px;
        float py;
    };

    // Core
    void CreateWindow(int w, int h, const char *str);

    void AppUpdate();
    void SetScene(Scene* scene);
    Scene* GetScene();

    // Graphic
    void GuiInit();
    void GuiDestroy();
    void GuiBegin();
    void GuiEnd();

    bool LoadTextureAtlas(const char *image_path, const char *ini_path, std::string AtlasKey);
    bool UnloadTextureAtlas(std::string AtlasKey);
    void DrawTextureAtlas(std::string AtlasKey, std::string key, int x, int y);
    void DrawTextureAtlasPro(std::string AtlasKey, std::string key, int x, int y, Color color, bool flipX = false, bool flipY = false);
    void DrawRenderTexture(const RenderTexture &buffer, const float &x, const float &y, const Color &color);
    // Tilemap
    // TilemapMetadata LoadTilemapMetadata(const char *file);
    // TilemapItem LoadTilemap(const char *file, TilemapMetadata metadata, Texture texture);

    // Utils
    Random &GetDefaultRandom();

    // Euclidean Distance
    double distance(const Vector2 &p1, const Vector2 &p2);
    double distanceSqrt(const Vector2 &p1, const Vector2 &p2);

    // Manhattan Distance
    double distanceManhattan(const Vector2 &p1, const Vector2 &p2);
}
