#pragma once

#include "utils/RaylibDeprecated.hpp"
#include <cmath>
#include <raylib.h>

class GridMap
{
public:
    GridMap()
    {
        Image image = GenImageColor(100, 100, WHITE);
        ImageDrawLine(&image, image.width*0.5f, 0, image.width*0.5f, image.height, {0, 0, 0, 120});
        ImageDrawLine(&image, 0, image.height*0.5f, 100, image.height*0.5f, {0, 0, 0, 120});
        ImageDrawRectangleLines(&image, {0.0f, 0.0f, image.width - 2.0f, image.height - 2.0f}, 2, {0, 0, 0, 180});

        texture = LoadTextureFromImage(image);
        UnloadImage(image);
    }

    ~GridMap()
    {
        UnloadTexture(texture);
    }


    void Draw(Camera2D camera)
    {
        Vector2 topLeft = GetScreenToWorld2D({0.0f, 0.0f}, camera);
        Vector2 botRight = GetScreenToWorld2D({(float)GetScreenWidth(), (float)GetScreenHeight()}, camera);

        float SWidth = (std::ceil((botRight.x-topLeft.x) / texture.width) + 1) * texture.width;
        float SHeight = (std::ceil((botRight.y-topLeft.y) / texture.height) + 1) * texture.height;

        float SX = (std::ceil(topLeft.x / texture.width) - 1) * texture.width;
        float SY = (std::ceil(topLeft.y / texture.height) - 1) * texture.height;

        DrawTextureTiled(
            texture,
            {0.0f, 0.0f, (float)texture.width, (float)texture.height},
            {SX, SY, SWidth, SHeight},
            // {topLeft.x, topLeft.y, botRight.x-topLeft.x, botRight.y-topLeft.y},
            {0.0f, 0.0f}, 0.0f, 1.0f, WHITE
        );
    }

private:
    Texture2D texture;
};